import os
import yaml
import pymysql

def get_config(path):
    return yaml.load(open(path).read())

def create_connection(config):
    connection = pymysql.connect(host=config['host'],
                                 user=config['user'],
                                 password=config['password'],
                                 db=config['db'])
    return connection

def check_column_exist(conn, db, table, column):
    cur = conn.cursor()
    cur.execute('''SELECT COUNT(*) FROM information_schema.columns WHERE
                        table_name='%s' AND
                        table_schema='%s' AND
                        column_name='%s' ''' % (table, db, column))
    return cur.fetchone()[0]

def add_column(conn, table, column, datatype):
    cur = conn.cursor()
    cur.execute('''ALTER TABLE %s ADD %s %s''' % (table, column, datatype))
    conn.commit()



from .util import get_config
from .util import create_connection

from .experiment import create_experiment
from .experiment import drop_experiment

from .instance import add_instance
from .instance import get_instance_id

from .method import alter_method
from .method import add_method
from .method import get_method_id

from .iteration import add_iteration
from .iteration import get_iteration



from .util import check_column_exist
from .util import add_column
import pymysql

def alter_method(conn, config):
    '''Alter the method table
    Purpose: to add more hyperparameters to the table

    Parameter:
        - conn (object): mysql connection
        - config (dict): specify the hyperparameter name and
                         data type
    '''

    # Add new fields if not exists in `method` table
    cur = conn.cursor()
    for column in config['method']:
        datatype = config['method'][column]
        if check_column_exist(conn, config['db'], 'method', column):
            if config['verbose']:
                print('[method|error] alter_method(%s, %s), reason: field existed' % (column, datatype))
        else:
            if config['verbose']:
                add_column(conn, 'method', column, datatype)
                print('[method|ok] alter_method(%s, %s)' % (column, datatype))
    # Add unique constraint for all fields
    fields = ','.join(config['method'].keys())
    query = '''ALTER TABLE method ADD UNIQUE unique_index (%s) ''' % (fields)
    print(query)
    cur.execute(query)
    conn.commit()

def add_method(conn, config, **kwargs):
    '''Add the method with hyperparameters

    Parameter:
        - conn (object): mysql connection
        - kwargs: <key> is hyperparameter name
                  corresponding <value> is the value
    '''
    cur = conn.cursor()
    fields = ','.join(kwargs.keys())
    values = []
    for _ in kwargs.values():
        if type(_) == str:
            values.append("'%s'" % _)
        else:
            values.append(str(_))
    values = ','.join(values)
    query = 'INSERT INTO method (%s) VALUES (%s)' % (fields, values)
    try:
        cur.execute(query)
        conn.commit()
    except pymysql.err.IntegrityError:
        if config['verbose']:
            print('[method|error] add_method(%s), reason: method duplicated' % values)
    else:
        if config['verbose']:
            print('[method|ok] add_method(%s)' % values)

def get_method_id(conn, config, **kwargs):
    cur = conn.cursor()
    values = []
    for key in kwargs.keys():
        value = kwargs[key]
        if type(value) == str:
            values.append("%s='%s'" % (key, value))
        else:
            values.append("%s=%s" % (key, str(value)))
    query = '''SELECT id FROM method WHERE %s''' % ' AND '.join(values)
    cur.execute(query)
    return cur.fetchone()[0]




















import yaml

def _create_default_instance_table(conn):
    cur = conn.cursor()
    cur.execute('''CREATE TABLE instance (
            id INT AUTO_INCREMENT PRIMARY KEY,
            name VARCHAR(512),
            kind VARCHAR(512))''')
    # Add unique constraint for all fields
    query = '''ALTER TABLE instance ADD UNIQUE unique_index (name, kind) '''
    cur.execute(query)
    conn.commit()

def _create_default_method_table(conn):
    cur = conn.cursor()
    cur.execute('''CREATE TABLE method(
            id INT AUTO_INCREMENT PRIMARY KEY)''')

def _create_default_iteration_table(conn):
    cur = conn.cursor()
    cur.execute('''CREATE TABLE iteration(
            id INT AUTO_INCREMENT PRIMARY KEY,
            method_id INT REFERENCES method(id),
            instance_id INT REFERENCES instance(id),
            best DOUBLE,
            mean DOUBLE,
            std DOUBLE,
            best_solution MEDIUMBLOB,
            num_iteration INT,
            num_evaluation INT,
            seed INT)''')
    query = 'ALTER TABLE iteration ADD UNIQUE unique_index (method_id, instance_id, num_iteration, seed)'
    cur.execute(query)
    conn.commit()

def create_experiment(conn, config):
    '''Create database and tables for the experiment.
    Create a table "experiment" that contain the experimental
    summarization

    Parameters:
        - conn (object): mysql connection
        - config (dict): contain db name
    '''
    cur = conn.cursor()
    cur.execute('CREATE DATABASE IF NOT EXISTS %s' % config['db'])
    conn.select_db(config['db'])
    _create_default_instance_table(conn)
    _create_default_method_table(conn)
    _create_default_iteration_table(conn)
    if config['verbose']:
        print('[experiment|ok] create_experiment(%s)' % config['db'])

def drop_experiment(conn, config):
    '''Drop database contain experimental results

    Parameters:
        - conn (object): mysql connection
        - config (dict): contain db name
    '''
    cur = conn.cursor()
    cur.execute('DROP DATABASE IF EXISTS %s' % config['db'])
    if config['verbose']:
        print('[experiment|ok] drop_experiment(%s)' % config['db'])



import pymysql

def add_instance(conn, config, name, kind):
    cur = conn.cursor()
    query = '''INSERT INTO instance (name, kind) VALUES ('%s', '%s')''' % (name, kind)
    try:
        cur.execute(query)
        conn.commit()
    except pymysql.err.IntegrityError:
        if config['verbose']:
            print('[instance|error] add_instance(%s, %s), reason: instance duplicated' % (name, kind))
    else:
        if config['verbose']:
            print('[instance|ok] add_instance(%s, %s)' % (name, kind))

def get_instance_id(conn, config, name, kind):
    cur = conn.cursor()
    query = '''SELECT id FROM instance WHERE name='%s' AND kind='%s' ''' % (name, kind)
    cur.execute(query)
    return cur.fetchone()[0]

def get_instance_by_id(conn, config, instance_id):
    cur = conn.cursor()
    query = 'SELECT * FROM instance WHERE id=%d' % instance_id
    cur.execute(query)
    return cur.fetchone()[0]



import pymysql

def add_iteration(conn, config, **kwargs):
    '''Add data from every iteration of each method for each instance

    Parameter:
        - conn (object): mysql connection
        - config (dict): configuration of experiment
        - kwargs:
            + method_id
            + instance_id
            + best
            + mean
            + std
            + best_solution
            + num_iteration
            + num_evaluation
            + seed
    '''
    cur = conn.cursor()
    fields = ','.join(kwargs.keys())
    values = []
    for _ in kwargs.values():
        if type(_) == str:
            values.append("'%s'" % _)
        else:
            values.append(str(_))
    values = ','.join(values)
    query = 'INSERT INTO iteration (%s) VALUES (%s)' % (fields, values)
    try:
        cur.execute(query)
        conn.commit()
    except pymysql.err.IntegrityError:
        if config['verbose']:
            print('[iteration|error] add_iteration(%s), reason: iteration duplicated' % values)
    else:
        if config['verbose']:
            print('[iteration|ok] add_iteration(%s)' % values)

def get_iteration(conn, config, **kwargs):
    '''Get data from every interation of each method for each instance

    Parameter:
        - conn (object): mysql connection
        - config (dict): configuration of experiment
        - kwargs:
            + method_id
            + instance_id
    '''
    cur = conn.cursor()
    values = []
    for key in kwargs:
        values.append('%s=%d' % (key, kwargs[key]))
    query = '''SELECT * FROM iteration WHERE %s ''' % ' AND '.join(values)
    cur.execute(query)
    for _ in cur.fetchall():
        yield _
























